@@ -14,7 +14,8 @@ U{http://www.opensource.org/licenses/bsd-license.php}
 import weakref, os.path
 import AEEvent, UIElement
 
-SINGLETONS = weakref.WeakValueDictionary()
+GLOBAL_SINGLETONS = weakref.WeakValueDictionary()
+APP_SINGLETONS = {}
 
 class AEChooser(UIElement.UIE):
   '''  
@@ -23,10 +24,15 @@ class AEChooser(UIElement.UIE):
   This class is abstract as most of its methods raise NotImplementedError and 
   need to be overriden in subclasses.
   
-  @cvar SINGLETON: Should only one instance of this chooser live at a time? 
-    This flag should be overriden by a subclass to indicate whether the
-    chooser is a singleton or not.
-  @type SINGLETON: boolean
+  @cvar GLOBAL_SINGLETON: Allow only one instance of a chooser to exist at a
+    time globally? This flag should be overriden by a subclass to indicate
+    whether the chooser is a global singleton or not.
+  @type GLOBAL_SINGLETON: boolean
+  @cvar APP_SINGLETON: Allow only one instance of a chooser to exist at a 
+    time for a given application? This flag should be overriden by a subclass 
+    to indicate whether the chooser is an application singleton or not.
+    Meaningless if L{GLOBAL_SINGLETON} is set.
+  @type APP_SINGLETON: boolean
   @cvar OK: Indicates the chooser is completing and its options should be
     applied
   @type OK: integer
@@ -47,7 +53,8 @@ class AEChooser(UIElement.UIE):
     method after a L{OK} or L{CANCEL} signal is sent
   @type block_signals: boolean
   '''
-  SINGLETON = False
+  GLOBAL_SINGLETON = False
+  APP_SINGLETON = False
   # these must be mutually exclusive from the signal constants used by subclass
   CANCEL = -1000
   APPLY = -1001
@@ -75,18 +82,31 @@ class AEChooser(UIElement.UIE):
     @param aid: Unique identifier for the application L{Tier} with which the 
       L{AEChooser} that fired this event is associated
     @type aid: L{Tier}
-    @raise ValueError: When the L{SINGLETON} flag is set to True and a 
-      singleton already exists
+    @raise ValueError: When the L{GLOBAL_SINGLETON} flag is set to True and a 
+      singleton already exists or L{APP_SINGLETON} flag is set to True and a
+      chooser for the application already exists.
     '''
     cls = self.getClassName()
-    if self.SINGLETON and cls in SINGLETONS:
-      raise ValueError(SINGLETONS[cls])
+    
+    if self.GLOBAL_SINGLETON and cls in GLOBAL_SINGLETONS:
+      raise ValueError(GLOBAL_SINGLETONS[cls])
+    elif (self.APP_SINGLETON and 
+          APP_SINGLETONS.has_key(cls) and 
+          aid in APP_SINGLETONS[cls].keys()):
+      raise ValueError(APP_SINGLETONS[cls][aid])
+    
     self.aid = aid
     self.event_man = event_man
     # store a weakref for singletons
-    if self.SINGLETON:
-      SINGLETONS[cls] = self
-  
+    if self.GLOBAL_SINGLETON:
+      GLOBAL_SINGLETONS[cls] = self
+    # store a WeakValueDictionary
+    elif self.APP_SINGLETON:
+      try:
+        APP_SINGLETONS[cls][aid] = self
+      except KeyError:
+        APP_SINGLETONS[cls] = weakref.WeakValueDictionary({aid:self})
+          
   def init(self, **kwargs):
     '''
     Initializes the chooser. Should enable the chooser for interaction with the
