@@ -46,7 +46,8 @@ DEFAULT_ASSOCIATIONS = {
            'MetacityPerk', 
            'GaimPerk', 
            'GTerminalPerk', 
-           'FirefoxPerk'],
+           'FirefoxPerk',
+           'SearchPerk'],
   'developer' :
           ['Keyboard', 
            'CliqueAudio', 
@@ -74,7 +75,8 @@ DEFAULT_ASSOCIATIONS = {
            'FirefoxPerk',
            'RawEventMonitor',
            'TaskMonitor', 
-           'IOMonitor'],
+           'IOMonitor',
+           'SearchPerk'],
   'monitors' :
           ['RawEventMonitor',
            'TaskMonitor', 
@@ -119,7 +121,8 @@ DEFAULT_ASSOCIATIONS = {
            'BasicSpeechPerk', 
            'BasicBraillePerk', 
            'AltShiftPerk', 
-           'GdmPerk'],
+           'GdmPerk',
+           'SearchPerk'],
   'mag' :
           ['Keyboard', 
            'GnomeMagDevice', 
