@@ -819,14 +819,15 @@ class RelRangeSetting(RelNumericSetting):
   '''
   Defines a range setting that specifies a value relative to another one. If 
   the parent style is None, acts just like a regular 
-  L{AEState.Setting.RangeSetting}.
+  L{RangeSetting}.
   '''
   pass
 
 class RelPercentRangeSetting(RelRangeSetting):
   '''
   Defines a percent setting that specifies a value relative to another one. If 
-  the parent style is None, acts just like a regular L{AEState.PercentSetting}.
+  the parent style is None, acts just like a regular
+  L{PercentRangeSetting}.
   
   The relative percentage of this setting is additive,
   '''
@@ -838,4 +839,4 @@ class RelPercentRangeSetting(RelRangeSetting):
     @return: A relative percent view of this range
     @rtype: L{RelPercentRangeView}
     '''
-    return RelPercentRangeView(self)
\ No newline at end of file
+    return RelPercentRangeView(self)
