@@ -263,8 +263,8 @@ class DefaultEventHandlerAdapter(Adapter):
     Creates an L{AEEvent.FocusChange} indicating that the accessible being
     adapted has lost the focus.
     
-    @param event: Source of the last raw focus change event
-    @type event: L{pyLinAcc.Accessible}
+    @param source: Source of the last raw focus change event
+    @type source: L{pyLinAcc.Accessible}
     @param kwargs: Parameters to be passed to any created L{AEEvent}
     @type kwargs: dictionary
     @return: L{AEEvent.FocusChange}
