@@ -7,13 +7,13 @@ interface on L{pyLinAcc.Accessibility.Accessible} objects.
 @author: Brett Clippingdale
 @author: Eirikur Hallgrimsson
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import pyLinAcc
 from POR import POR
@@ -63,18 +63,17 @@ class DefaultAccInfoAdapter(PORAdapter):
     '''
     Gets a list of name:value attribute pairs for the subject.
     
-    @return: Name:value pairs of the accessible object
-    @rtype: list of strings
+    @return: Name:value pairs of the object attributes
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     @raise NotImplementedError: When this accessible does not support object
       attributes
     '''
     try:
-      attribute_string = self.accessible.getAttributes()
-    except (AttributeError, Constants.NotImplemented):
+      attrs = self.accessible.getAttributes()
+    except (AttributeError, Constants.NotImplemented,Constants.CORBAException):
       raise NotImplementedError
-    attribute_list = attribute_string.split('; ')
-    return attribute_list
+    return dict([attr.split(':', 1) for attr in attrs])
 
   @pyLinAcc.errorToLookupError
   def getAccFloatValueExtents(self):
@@ -118,20 +117,22 @@ class DefaultAccInfoAdapter(PORAdapter):
     Gets all of the default text attributes assigned to the subject.
     
     @return: Name:value pairs of the default text attributes
-    @rtype: list of strings
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     @raise NotImplementedError: When this accessible does not support the text
       interface
     '''
     text = Interfaces.IText(self.accessible)
     try:
-      val = text.getDefaultAttributeSet()
+      attrs = text.getDefaultAttributeSet()
+      if attrs is None:
+        return {}
     except (AttributeError, Constants.NotImplemented):
       val = text.getDefaultAttributes()
-    if val is '':
-      return ''
-    attrs = val.split('; ')
-    return attrs
+      if val is '':
+        return {}
+      attrs = val.split('; ', 1)
+    return dict([attr.split(':', 1) for attr in attrs])
   
   @pyLinAcc.errorToLookupError
   def getAccTextSelectionCount(self):
@@ -257,14 +258,39 @@ class DefaultAccInfoAdapter(PORAdapter):
     Gets the visual position of the focal point wihin an accessible object,
     the center in a simple box-like control.
 
+    @return: x,y coordinate
+    @rtype: 2-tuple integer
+    @raise NotImplementedError: When this accessible does not support the
+      Component interface
+    '''
+    try:
+      text = Interfaces.IText(self.accessible)
+    except NotImplementedError:
+      text = None
+      
+    if text:
+      # use the character extents to track to the current character
+      off = self.char_offset
+      x, y, w, h = text.getCharacterExtents(off, Constants.DESKTOP_COORDS)
+      return x, y
+    else:
+      # just center on the widget
+      c = Interfaces.IComponent(self.accessible)
+      bb = c.getExtents(Constants.DESKTOP_COORDS)
+      return bb.x+bb.width/2, bb.y+bb.height/2
+  
+  @pyLinAcc.errorToLookupError
+  def getAccPosition(self):
+    '''
+    Gets the position of the accessible object, usually upper-left corner.
+    
     @return: x,y coordinate
     @rtype: 2-tuple integer
     @raise NotImplementedError: When this accessible does not support the
       Component interface
     '''
     c = Interfaces.IComponent(self.accessible)
-    bb = c.getExtents(Constants.DESKTOP_COORDS)
-    return (bb.x+bb.width)/2, (bb.y+bb.height)/2
+    return c.getPosition(Constants.DESKTOP_COORDS)
 
   @pyLinAcc.errorToLookupError
   def getAccTextAttr(self, name):
@@ -286,11 +312,7 @@ class DefaultAccInfoAdapter(PORAdapter):
       return val
     except (AttributeError, Constants.NotImplemented):
       attrs = self.getAccAllTextAttrs()
-      for s in attrs:
-        n, v = s.split(':')
-        if n == name:
-          return v
-      return ''
+      return attrs.get(name, '')
   
   @pyLinAcc.errorToLookupError 
   def getAccIndex(self):
@@ -370,7 +392,7 @@ class DefaultAccInfoAdapter(PORAdapter):
     Gets all the text attributes of a given accessible.
     
     @return: Name:value pairs of the text attributes at the character offset
-    @rtype: list of strings
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     @raise NotImplementedError: When this accessible does not support the text
       interface
@@ -378,17 +400,24 @@ class DefaultAccInfoAdapter(PORAdapter):
     text = Interfaces.IText(self.accessible)  
     # use item_offset of zero if None
     if self.item_offset is None:
-      self.item_offset = 0
+      io = 0
+    else:
+      io = self.item_offset
     # get the attributes at the given item_offset (start of a line) plus 
     # char_offset
+    index = io+self.char_offset
     try:
+      raise AttributeError      
       # do not get default attributes
-      val, start, end = text.getAttributeRun(self.item_offset+self.char_offset,
-                                             False)
+      attrs, start, end = text.getAttributeRun(index, False)
+      if attrs is None:
+        return {}
     except (AttributeError, Constants.NotImplemented):
-      val, start, end = text.getAttributes(self.item_offset + self.char_offset)
-    attrs = val.split('; ')
-    return attrs
+      val, start, end = text.getAttributes(index)
+      if val is '':
+        return {}
+      attrs = val.split('; ', 1)
+    return dict([attr.split(':', 1) for attr in attrs])
   
   @pyLinAcc.errorToLookupError
   def getAccSelection(self):
@@ -471,7 +500,7 @@ class DefaultAccInfoAdapter(PORAdapter):
     @raise LookupError: When the accessible or any parent accessible up to the
       application is dead
     '''
-    app = self.accessible.getApplication()
+    app = Interfaces.IApplication(self.accessible.getApplication())
     return app.getLocale(Constants.LOCALE_TYPE_MESSAGES)
 
   @pyLinAcc.errorToLookupError
@@ -485,6 +514,18 @@ class DefaultAccInfoAdapter(PORAdapter):
     @raise LookupError: When the accessible object is dead
     '''
     return unicode(self.accessible.getLocalizedRoleName(), 'utf-8')
+  
+  @pyLinAcc.errorToLookupError
+  def getAccRole(self):
+    '''
+    Gets the accessible role of the subject. Ignores the given item offset 
+    because the default adapter assumes there are no items.
+    
+    @return: Unlocalized role
+    @rtype: string
+    @raise LookupError: When the accessible object is dead
+    '''
+    return self.accessible.getRoleName()
     
   @pyLinAcc.errorToLookupError
   def getAccName(self):
@@ -569,6 +610,7 @@ class DefaultAccInfoAdapter(PORAdapter):
     @rtype: list
     @raise LookupError: When the accessible object is dead
     '''
+    role = self.accessible.getRoleName()
     states = set([pyLinAcc.stateToString(val) for val in 
                   self.accessible.getState().getStates()])
     # collapsed isn't always set properly, so add it if expandable but not
@@ -576,11 +618,12 @@ class DefaultAccInfoAdapter(PORAdapter):
     if 'expandable' in states and 'expanded' not in states:
       states.add('collapsed')
     # give some indicator of not being enabled
-    if 'enabled' not in states and self.getAccRoleName() != 'icon':
+    if ('enabled' not in states and role != 'icon' and 
+        ('focusable' in states or 'editable' in states or 
+         'selectable' in states)):
       states.add('disabled')
     # give an indication of not checked
-    if (self.accessible.getRoleName().find('check') > -1 and 
-        'checked' not in states):
+    if (role.find('check') > -1 and 'checked' not in states):
       states.add('unchecked')
     return states
   
@@ -704,6 +747,20 @@ class DefaultAccInfoAdapter(PORAdapter):
     '''
     return role == self.accessible.getRoleName()
 
+  @pyLinAcc.errorToLookupError
+  def hasAccOneRole(self, *roles):
+    '''
+    Gets if the subject has any one of the given roles. The given role strings
+    are compared to the unlocalized string role of the accessible.
+    
+    @param roles: Names of the roles (e.g. 'terminal', 'glass pane')
+    @type roles: list of string
+    @return: Does the accessible have any one of the given roles?
+    @rtype: boolean
+    '''
+    return self.accessible.getRoleName() in roles
+    
+
   @pyLinAcc.errorToLookupError
   def getAccActionNames(self):
     '''
@@ -747,9 +804,6 @@ class DefaultAccInfoAdapter(PORAdapter):
     key bindings into tuples since some actions can have multiple ways of 
     activating them dependent on the context.
     
-    @todo: PP: improve so that key bindings that have a ; in them are parsed
-      properly
-    
     @return: List of tuples, each containing one or more string keysym names
       indicating the keys to press in sequence to perform the action
     @rtype: list of tuple of string
