@@ -121,9 +121,6 @@ def _getEmbedCharOffset(acc, index):
   '''
   Gets the character offset of the embedded character at the given index.
   
-  @todo: PP: this method can be optimized to find location of the nth embed
-    character once the new hyperlink interface becomes available
-  
   @param acc: Accessible which should be searched for embed characters
   @type acc: L{pyLinAcc.Accessible}
   @param index: Index of the embedded character in the count of all embedded
@@ -416,7 +413,7 @@ class HypertextAccInfoAdapter(DefaultAccInfoAdapter):
     @raise LookupError: When the accessible object is dead
     '''
     if self.item_offset is None:
-      return u''
+      return self.accessible.name
     it = Interfaces.IText(self.accessible)
     # get the current line start and end offsets
     text, lstart, lend = \
