@@ -6,7 +6,7 @@ Table interface but not the Selection interface.
 @author: Peter Parente
 @author: Eirikur Hallgrimsson
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -180,30 +180,36 @@ class TableNavAdapter(DefaultNavAdapter):
     return Interfaces.ITable(acc)
   
   @pyLinAcc.errorToLookupError
-  def _getVisibleItemExtents(self):
+  def _getVisibleItemExtents(self, only_visible):
     '''
-    Gets the item offsets of the first and last visible items in a 1D list of 
-    items.
+    Gets the item offsets of the first and last items in a table of cells.
     
+    @param only_visible: Only consider the first and last cells visible in
+      the table (True) or the absolute first and last cells (False)?
+    @type only_visible: boolean
     @return: First and last item offsets
     @rtype: 2-tuple of integer
     @raise LookupError: When the first or last item or parent accessible is
       not available
     '''
     acc = self.accessible
-    comp = Interfaces.IComponent(acc)
-    e = comp.getExtents(Constants.WINDOW_COORDS)
-    # get the first item
-    x, y = e.x+FUDGE_PX, e.y+FUDGE_PX
-    try:
-      first = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
-    except TypeError:
+    if only_visible:
+      comp = Interfaces.IComponent(acc)
+      e = comp.getExtents(Constants.WINDOW_COORDS)
+      # get the first item
+      x, y = e.x+FUDGE_PX, e.y+FUDGE_PX
+      try:
+        first = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+      except TypeError:
+        first = None
+      # get the last item
+      x, y = e.x+e.width-FUDGE_PX, e.y+e.height-FUDGE_PX
+      try:
+        last = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+      except TypeError:
+        last = None
+    else:
       first = None
-    # get the last item
-    x, y = e.x+e.width-FUDGE_PX, e.y+e.height-FUDGE_PX
-    try:
-      last = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
-    except TypeError:
       last = None
     # compute indices
     if first:
@@ -223,8 +229,6 @@ class TableNavAdapter(DefaultNavAdapter):
     Gets the next item relative to the one indicated by the L{POR}
     providing this interface.
 
-    Currently, this assumes only_visible is True.
-
     @param only_visible: True when Item in the returned L{POR} must be visible
     @type only_visible: boolean
     @return: Point of regard to the next item in the same accessible
@@ -236,7 +240,7 @@ class TableNavAdapter(DefaultNavAdapter):
     acc = self.accessible
     off = self.item_offset
     # get the first and last indices
-    i, j = self._getVisibleItemExtents()
+    i, j = self._getVisibleItemExtents(only_visible)
     if off is None or off < i:
       # return the first visible, non-header item
       if not IAccessibleInfo(POR(acc.getChildAtIndex(i))).isAccVisible():
@@ -267,8 +271,6 @@ class TableNavAdapter(DefaultNavAdapter):
     Gets the previous item relative to the one indicated by the L{POR} providing
     this interface. 
     
-    Currently, this assumes only_visible is True.
-    
     @param only_visible: True when Item in the returned L{POR} must be visible
     @type only_visible: boolean
     @return: Point of regard to the previous item in the same accessible
@@ -281,7 +283,7 @@ class TableNavAdapter(DefaultNavAdapter):
     off = self.item_offset
     comp = Interfaces.IComponent(acc)
     # get the first and last indices
-    i, j = self._getVisibleItemExtents()
+    i, j = self._getVisibleItemExtents(only_visible)
     # and check if the first item is visible since it might be a header
     if off is None:
       # no more visible items
@@ -328,7 +330,7 @@ class TableNavAdapter(DefaultNavAdapter):
     if IAccessibleInfo(POR(child)).isAccVisible() or not only_visible:
       return POR(acc, acc.childCount-1, 0)
     # use coords to get the last visible item
-    i, j = self._getVisibleItemExtents()
+    i, j = self._getVisibleItemExtents(only_visible)
     return POR(acc, j, 0)
     
   @pyLinAcc.errorToLookupError
@@ -351,7 +353,7 @@ class TableNavAdapter(DefaultNavAdapter):
     if IAccessibleInfo(POR(child)).isAccVisible() or not only_visible:
       return POR(acc, 0, 0)
     # use coords to get the first visible item
-    i, j = self._getVisibleItemExtents()
+    i, j = self._getVisibleItemExtents(only_visible)
     return POR(acc, i, 0)
   
   @pyLinAcc.errorToLookupError 
