@@ -43,7 +43,7 @@ def _multiLineCondition(acc):
     # terminals and paragraphs are always multiline
     # note: this does not account for paragraphs that implement hypertext
     return True
-  elif r in (c.ROLE_PAGE_TAB, c.ROLE_LABEL):
+  elif r in (c.ROLE_PAGE_TAB, c.ROLE_LABEL, c.ROLE_EDITBAR):
     # these widgets sometimes show as multiline, but are succinct enough to
     # treat as single line
     return False
@@ -61,7 +61,7 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
   as items. Expects the subject to be a L{POR}.
   
   Adapts accessibles that have a role of terminal or have a state of multiline, 
-  single line, or editable and provide the L{Interfaces.IText} interface. 
+  single line, or editable and provide the Text interface. 
   Does not adapt L{POR} accessibles with role of page tab.
   '''
   provides = [IAccessibleInfo]
@@ -162,7 +162,7 @@ class TextNavAdapter(DefaultNavAdapter):
   in the L{IAccessibleNav} interface. 
   
   Adapts accessibles that have a role of terminal or have a state of multiline, 
-  single line, or editable and provide the L{Interfaces.IText} interface. 
+  single line, or editable and provide the Text interface. 
   Does not adapt L{POR} accessibles with role of page tab.
   '''
   provides = [IItemNav, IAccessibleNav]
@@ -377,7 +377,7 @@ class SimpleTextEventHandlerAdapter(DefaultEventHandlerAdapter):
   events for caret and text events. Expects the subject to be a
   L{pyLinAcc.Accessible}.
   
-  Adapts subject accessibles that provide the L{Interfaces.IText} interface.
+  Adapts subject accessibles that provide the Text interface.
   '''
   provides = [IEventHandler]
   
@@ -515,9 +515,16 @@ class SimpleTextEventHandlerAdapter(DefaultEventHandlerAdapter):
     por = POR(self.subject, 0, 0)
     char_offset = text.caretOffset
     
-    # create a focus event for this POR
-    # and add a caret event to indicate the current position
-    return (FocusChange(por, True, **kwargs),    
+    # GTK and Gecko (Firefox) toolkits handle caret events differently.
+    # GTK fails to produce a caret event while Gecko produces one for you.
+    # This code branches on the toolkit type and returns the required events
+    # for each toolkit type.
+    if self.subject.getApplication().toolkitName == 'Gecko':
+      return (FocusChange(por, True, **kwargs),)
+    else:
+      # create a focus event for this POR
+      # and add a caret event to indicate the current position
+      return (FocusChange(por, True, **kwargs),    
             CaretChange(por, unicode(line, 'utf-8'), char_offset, **kwargs))
 
 class TextEventHandlerAdapter(SimpleTextEventHandlerAdapter):
@@ -527,7 +534,7 @@ class TextEventHandlerAdapter(SimpleTextEventHandlerAdapter):
   L{pyLinAcc.Accessible}.
   
   Adapts subject accessibles that have a role of terminal or have a state of 
-  multiline, single line, or editable and provide the L{Interfaces.IText} 
+  multiline, single line, or editable and provide the Text
   interface. Does not adapt accessibles with role of page tab.
   '''
   provides = [IEventHandler]
