@@ -5,7 +5,7 @@ L{Tier} at runtime.
 @author: Brett Clippingdale
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -51,6 +51,7 @@ class PerkChooser(AEChooser.AEChooser):
   LOWER = LOWER
   TO_LOAD = TO_LOAD
   TO_UNLOAD = TO_UNLOAD
+  APP_SINGLETON = True
 
   def init(self, app_name, loaded, unloaded, timestamp, **kwargs):
     '''
@@ -70,7 +71,6 @@ class PerkChooser(AEChooser.AEChooser):
     '''
     # initialize the view to None
     self.uie_view = None
-
     # load the glade file
     gtk.glade.set_custom_handler(self._createCustomWidget, loaded, unloaded)
     source = gtk.glade.XML(self._getResource('perk_chooser.glade'), 
@@ -80,11 +80,6 @@ class PerkChooser(AEChooser.AEChooser):
     # get the dialog widget
     self.dialog = source.get_widget('main dialog')
     self.dialog.set_title(self.dialog.get_title() % app_name)
-    try:
-      # try to bring the window to the foreground if this feature is supported
-      self.dialog.window.set_user_time(timestamp)
-    except AttributeError:
-      pass
     
     # set the data
     self.uie_view.setData(loaded, unloaded)
@@ -92,6 +87,18 @@ class PerkChooser(AEChooser.AEChooser):
     # connect all signal handlers
     source.signal_autoconnect(self)
     
+    self.activate(timestamp, present=False)
+    
+  def activate(self, timestamp, present=True, **kwargs):
+    '''Try to bring the window to the foreground.'''
+    try:
+      # try to bring the window to the foreground if this feature is supported
+      self.dialog.window.set_user_time(timestamp)
+    except AttributeError:
+      pass
+    if present:
+      self.dialog.present()
+    
   def _createCustomWidget(self, source, name, func, *args):
     '''
     Creates an ordered UIE chooser widget.
