@@ -125,8 +125,6 @@ class DeviceManager(object):
     continue to exist until the L{DeviceManager} has new devices available 
     for I/O.
     
-    @todo: PP: unload input devices too once input device modifiers can be
-      properly saved and restored
     '''
     for d in self.out_devs[:]:
       self.temp_devs.append(d)
@@ -154,8 +152,8 @@ class DeviceManager(object):
     
     This is a generic method which receives all content and commands to be 
     rendered and executed on a device. Standard name identifiers should be 
-    used whenever possible. For instance, L{AEConstants.CMD_STOP} and
-    L{AEConstants.CMD_TALK}. However, device specific names and values
+    used whenever possible. For instance, L{AEConstants.Output.CMD_STOP} and
+    L{AEConstants.Output.CMD_TALK}. However, device specific names and values
     are certainly possible.
 
     @param name: Descriptor of the data value sent
@@ -280,8 +278,6 @@ class DeviceManager(object):
   def sendString(self, dev, text, sem, layer, por=None):
     '''
     Sends the string to the specified output device.
-    
-    @todo: PP: send indices and collect word markers
 
     @param dev: Device that should receive the command
     @type dev: L{AEOutput}
@@ -308,7 +304,6 @@ class DeviceManager(object):
     parsed = dev.parseString(text, style, por)
     # send all words, pors, and styles
     for word, por, new_style in parsed:
-      # @todo: PP: send the index and collect the marker
       dev.send(AEConstants.CMD_STRING, word, new_style)
       # mark the style as clean
       new_style.makeClean()
@@ -426,9 +421,7 @@ class DeviceManager(object):
     '''
     Unregisters the given input device. Removes this manager from
     the list of gesture listeners stored in the device.
-    
-    @todo: PP: let an ouput device save its state too?
-    
+        
     @param dev: Device to unregister
     @type dev: L{AEInput.AEInput}
     '''
@@ -519,11 +512,11 @@ class DeviceManager(object):
   
   def _initOutputStyles(self, dev):
     '''
-    Initializes styles for an L{AEOutput} device. Calls 
-    L{AEOutput.AEOutput.createDistinctStyles} on the device to get an initial 
-    batch of styles to use to distinguish some types of information. If that
-    method is not implemented, the exception is ignored. Future requests to
-    use styles per semantic tag will resort to making flyweights for the
+    Initializes styles for an L{AEOutput} device. Calls
+    L{AEOutput.Base.AEOutput.createDistinctStyles} on the device to get an
+    initial batch of styles to use to distinguish some types of information. If
+    that method is not implemented, the exception is ignored. Future requests
+    to use styles per semantic tag will resort to making flyweights for the
     default style on the device.
     
     @param dev: Device reference
@@ -690,9 +683,6 @@ class DeviceManager(object):
                       dev=gesture.getDevice())
     
   def _onIndex(self, dev, index):
-    '''
-    @todo: PP: handle index callback markers
-    '''
     pass
     # notify monitors
     #self.out_mons.show(event=OutputEvent.MARKER)
