@@ -27,19 +27,21 @@ __uie__ = dict(kind='device')
 MAX_KEY_CHORD = 1
 log = logging.getLogger('BrlAPIDevice')
 
-def _keyCodeToKeyName(code):
+def _keyCodeToKeyName(command):
   '''
   Gets the name of the given key code by using the brlapi's module dictionary
-  to lookup its keycmd value and return its keycmd name. If the code is not
+  to lookup its command value and return its command name. If the code is not
   known, an empty string is returned.
   
-  @param code: Hardware keycode
-  @type code: integer
+  @note: brlapi delivers commands in the tuple (type, command, argument, flags)
+  
+  @param command: Hardware keycode command
+  @type command: integer
   @return: Name of the key
   @rtype: string
   '''
   for name, value in brlapi.__dict__.items():
-      if code == value:
+      if command == value:
         return name
   return ''
 
@@ -156,7 +158,8 @@ class BrlAPIDevice(AEOutput.Braille, AEInput.AEInput):
     # tell brlapi to not send me any key_cmds.  Each task will register for key
     
     # TODO fix when brltty API settles down
-    #self.brlconn.ignoreKeyRange([0, brlapi.KEY_MAX])
+   # self.brlconn.ignoreKeyRange([0, brlapi.KEY_MAX])
+    self.brlconn.ignoreAllKeys()
     
     # start input event manager
     self.inputmngr = BrlInputEventListener((self))
@@ -209,8 +212,8 @@ class BrlAPIDevice(AEOutput.Braille, AEInput.AEInput):
     except brlapi.OperationError:
       log.warn('braille input failed')
       return
-   # print "Key %ld (%x %x %x %x) !" % (key, k["type"], k["command"], k["argument"], k["flags"])
-   # print "keycode=", _keyCodeToKeyName(key)
+    print "Key %ld (%x %x %x %x) !" % (key, k["type"], k["command"], k["argument"], k["flags"])
+    print "keycode=", _keyCodeToKeyName(k["command"])
     
     gesture = AEInput.Gesture(self)
     gesture.addActionCode(k['command'])
@@ -259,21 +262,13 @@ class BrlAPIDevice(AEOutput.Braille, AEInput.AEInput):
     @type codes: list
     @raise NotImplementedError: When this method is not overridden by a subclass
     '''
-    # TODO remove when brltty API settles down
-    if True:
-      return
-    
     for codelist in codes:
       for code in codelist:
         if self.registered_keys.has_key(code):
           self.registered_keys[code] += 1
         else:
           self.registered_keys[code] = 1
-          if code == brlapi.KEY_CMD_ROUTE:
-            self.brlconn.acceptKeyRange([brlapi.KEY_CMD_ROUTE, \
-                            brlapi.KEY_CMD_ROUTE|brlapi.KEY_CMD_ARG_MASK])
-          else:
-            self.brlconn.acceptKeySet([code])
+          self.brlconn.acceptKeys(brlapi.rangeType_command,[brlapi.KEY_TYPE_CMD|code])
             
   def removeKeyCmd(self, codes):
     ''' 
@@ -283,21 +278,13 @@ class BrlAPIDevice(AEOutput.Braille, AEInput.AEInput):
     @type codes: list
     @raise NotImplementedError: When this method is not overridden by a subclass
     '''
-    # TODO remove when brltty API settles down
-    if True:
-      return
-    
     for codelist in codes:
       for code in codelist:
         try:
           self.registered_keys[code] -= 1
           if self.registered_keys[code] == 0:
             del self.registered_keys[code]
-            if code == brlapi.KEY_CMD_ROUTE:
-              self.brlconn.ignoreKeyRange([brlapi.KEY_CMD_ROUTE, \
-                            brlapi.KEY_CMD_ROUTE|brlapi.KEY_CMD_ARG_MASK])
-            else:
-              self.brlconn.ignoreKeySet([code])
+            self.brlconn.ignoreKeys(brlapi.rangeType_command,[brlapi.KEY_TYPE_CMD|code])
         except KeyError:
           pass
   
@@ -465,8 +452,14 @@ class BrlAPIDevice(AEOutput.Braille, AEInput.AEInput):
     @rtype: tuple
     @raise NotImplementedError: When not overridden in a subclass
     '''
-    left = len(self.EllipsisStyles[self.default_style.EllipsisStyle][0])
-    right = len(self.EllipsisStyles[self.default_style.EllipsisStyle][1])
+    left = 0
+    right = 0
+    
+    if self.default_style.EllipsisLeft:
+      left = len(self.EllipsisStyles[self.default_style.EllipsisStyle][0])
+    if self.default_style.EllipsisRight:
+      right = len(self.EllipsisStyles[self.default_style.EllipsisStyle][1])
+   
     return (left, right)
     
   def sendGetMissingCellCount(self):
