@@ -448,8 +448,8 @@ class IAccessibleInfo(Interface):
     '''
     Gets a list of name:value attribute pairs for the subject.
     
-    @return: Name:value pairs of the accessible object
-    @rtype: list of strings
+    @return: Name:value pairs of the accessible object attributes
+    @rtype: dictionary
     @raise LookupError: When the accessible object is dead
     '''
     pass
