@@ -190,8 +190,8 @@ class Perk(Task.Tools.All, UIElement.UIE):
 
   def _initTask(self, task):
     '''
-    Initializes a L{Task} by calling L{Task.Task.preInit} and 
-    L{Task.Task.init} if it has not already been initialized.
+    Initializes a L{Task} by calling L{Task.Base.Task.preInit} and 
+    L{Task.Base.Task.init} if it has not already been initialized.
     
     @param task: L{Task} to initialize
     @type task: L{Task}
@@ -624,9 +624,6 @@ class Perk(Task.Tools.All, UIElement.UIE):
     
     Invokes L{Tier.Tier.removeChainRef} to delete a reference to this L{Perk}
     to the L{Tier} which will later have to traverse the chain across L{Perk}s.
-    
-    @todo: PP: call tier.removeChainRef when there are no more links for the
-      target in this Perk
 
     @param name: Name of the L{Task} to link
     @type name: string
@@ -690,4 +687,4 @@ class Perk(Task.Tools.All, UIElement.UIE):
     @return: All names
     @rtype: list of string
     '''
-    return self.named_tasks.keys()
\ No newline at end of file
+    return self.named_tasks.keys()
