@@ -69,11 +69,17 @@ class AltShiftPerk(Perk.Perk):
       self.registerCommand(kbd, 'cycle text attributes',False,pair+[kbd.AEK_B])
       self.registerCommand(kbd, 'read description', False, pair+[kbd.AEK_D])
       self.registerCommand(kbd, 'read all', False, pair+[kbd.AEK_H])
+      self.registerCommand(kbd, 'mouse to pointer', False, pair+[kbd.AEK_X])
       
       # default dialog perk
       self.registerCommand(kbd, 'show perk chooser', False, pair+[kbd.AEK_A])
       self.registerCommand(kbd, 'show settings chooser', False, 
                            pair+[kbd.AEK_Q])
+                           
+      # search perk
+      self.registerCommand(kbd, 'search show chooser', False, pair+[kbd.AEK_S])
+      self.registerCommand(kbd, 'search find prev', False, pair+[kbd.AEK_W])
+      self.registerCommand(kbd, 'search find next', False, pair+[kbd.AEK_E])
 
   def getDescription(self):
     return _('Defines hotkeys for functions in basic speech, Braille, review, '
