@@ -19,8 +19,6 @@ from POR import POR
 from i18n import bind, _
 from AEConstants import *
 
-# TODO:  remove.  for debug only
-import time
 
 # metadata describing this Perk
 __uie__ = dict(kind='perk', tier=None, all_tiers=True)
@@ -38,6 +36,8 @@ class BasicBraillePerkState(Perk.PerkState):
   SnapOnUpdate (bool): snap braille display to previous location upon update?
   
   SnapToWord (bool): snap braille display to nearest word upon paging?
+  @ivar BrailleAvailable: Is Braille device available?
+  @type BrailleAvailable: boolean
   '''
   def init(self):
     '''
@@ -63,6 +63,8 @@ class BasicBraillePerkState(Perk.PerkState):
     self.newBool('BrailleReview', False, _('Track review keys?'), 
                 _('When set, reviewing is tracked by the Braille display.  '
                   'Otherwise, display is not updated when reviewing.'))
+    self.BrailleAvailable = True
+    
    
   def getGroups(self):
     '''
@@ -120,7 +122,7 @@ class BasicBraillePerk(Perk.Perk):
     self.abscaretpos = -1
     self.current_text = ""
     
-    # set the output device                       
+    # set the output device      
     self.setPerkIdealOutput('braille')
     
     # register braille events handlers
@@ -142,11 +144,19 @@ class BasicBraillePerk(Perk.Perk):
     self.registerTask(HandleHome('braille home'))
     self.registerTask(HandleReview('braille review'))
     self.registerTask(HandlePointerToFocus('braille pointer to por'))
-    self.registerTask(BrailleItemText('braille item text'))
-                         
+    self.registerTask(BrailleText('braille text'))
+                       
     #register braille device commands
-    #TODO: send audible error
-    brlindev = self.getInputDevice(None, 'braille')
+    try:
+      brlindev = self.getInputDevice(None, 'braille')
+    except:
+      if self.state.BrailleAvailable:
+        self.stopNow()
+        self.doTask('read message', text=_('braille device unavailable'), 
+                     sem=SEM_ERROR)
+        self.state.BrailleAvailable = False
+      return
+      
     
     self.registerCommand(brlindev, 'braille scroll backward', True, 
                          [brlindev.KEY_CMD_FWINLT])
@@ -220,7 +230,6 @@ class BasicBraillePerk(Perk.Perk):
     else:
       return max(-1, self.abscaretpos - self.getLeftSlice(totalcells)) 
      
-  # TODO:  move to Output text as private method
   def getLeftSlice(self, totalcells):
     '''
     Get the leftmost slice indice for the current braille output
@@ -380,12 +389,9 @@ class HandlePageForward(Task.InputTask):
       self.perk.scroll_offset = ctlen + state.RightPadding\
              - style.TotalCells - self.perk.pre_left_slice
       
-    # look at the current left slice position and move backward until space
-    # or end is found.  TODO:  move to parse word similar to speech.
     if state.SnapToWord:
       found = False
       wordadj = 0
-      # TODO change hardcoded len(ellipsis)
       for i in xrange(self.perk.scroll_offset+ \
                      self.perk.pre_left_slice+l_el_len, -1, -1):
         if self.perk.current_text[i] == ' ' or i == 0:
@@ -419,8 +425,6 @@ class HandlePageBackward(Task.InputTask):
     else:
       self.perk.scroll_offset = (self.perk.pre_left_slice * -1)
     
-    # look at the current left slice position and move backward until space
-    # or end is found.  TODO:  move to parse word similar to speech.
     if state.SnapToWord:
       found = False
       wordadj = 0
@@ -630,12 +634,11 @@ class HandleReview(Task.InputTask):
           self.perk.scroll_offset = 0
         # get the por at the focus
         focus_por = self.getAccFocus()
+        # set caret position
         if focus_por is not None and (focus_por.item_offset == self.task_por.item_offset):
           self.perk.setAbsCaretPos(focus_por.char_offset)
         else:
           self.perk.setAbsCaretPos(-1)
-    
-        self.perk.setAbsCaretPos(self.task_por.char_offset)
           
         if self.task_por.char_offset == 0:
           self.perk.pre_left_slice = self.task_por.char_offset
@@ -652,8 +655,6 @@ class HandlePointerToFocus(Task.InputTask):
   '''
   Synchronizes the Braille display with the text at the focus when the pointer
   is warped back to the caret.
-  
-  @todo: PP: render caret position when appropriate
   '''
   def execute(self, **kwargs):
     # update current text
@@ -663,14 +664,16 @@ class HandlePointerToFocus(Task.InputTask):
     # now Braille it
     self.doTask('braille current text')
     
-class BrailleItemText(Task.Task):
+class BrailleText(Task.Task):
   '''
   Outputs item text to Braille display.
-  
   '''
-  def execute(self, **kwargs):
+  def execute(self, text=None, **kwargs):
     # update current text
-    self.perk.current_text = self.getItemText()
+    if text is None:
+      self.perk.current_text = self.getItemText()
+    else:
+      self.perk.current_text = text
     if self.perk_state.SnapOnUpdate:
       self.perk.scroll_offset = 0
     # now Braille it
