@@ -12,8 +12,8 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 
-Current revision $Revision: 584 $
-Latest change by $Author: shaeger $ on $Date: 2007-02-23 12:55:48 -0500 (Fri, 23 Feb 2007) $
+Current revision $Revision: 705 $
+Latest change by $Author: parente $ on $Date: 2007-04-10 22:18:43 -0400 (Tue, 10 Apr 2007) $
 '''
 # import useful modules for Perks
 import Perk, Task, AEConstants
@@ -28,10 +28,8 @@ class ZoomMove(object):
   '''
   Class that deals with seamless panning and repositioning of the magnifier.
 
-  @cvar pan_rate: Duation in milliseconds between "frames"
+  @cvar pan_rate: Duration in milliseconds between "frames"
   @type pan_rate: int
-  @cvar timeout_id: GObject timeout id.
-  @type timeout_id: int
   @cvar end_coord: Coordinate that we are trying to get to
   @type end_coord: tuple
   @cvar last_track_type: The last type of tracking we did
@@ -44,7 +42,6 @@ class ZoomMove(object):
   @type smooth_pan: boolean
   '''
   pan_rate = 40
-  timeout_id = None
   smooth_pan = None
   accel = None
   velocity = None
@@ -353,14 +350,21 @@ class BasicMagPerk(Perk.Perk, ZoomMove):
       account so that the magnifier doesn't jerk?
     @type track_mouse: boolean
     '''
-    # get the extents and position of the POR
-    por_width, por_height = self.getAccVisualExtents(por)
-    pos_x, pos_y = self.getAccPosition()
+    # get the extents of the POR
+    try:
+      por_width, por_height = self.getAccVisualExtents(por)
+    except TypeError:
+      # In case the extents of the POR can't be determined, the mouse pointer
+      # position is not taken into account when calculating the desired focal
+      # point, since we can't evaluate if it is within the POR.
+      track_mouse = False
+    # get the position of the POR
+    pos_x, pos_y = self.getAccPosition() 
     # get the last known mouse position
     m_x, m_y = self.perk.mousepos
     # get the recommended focal point for the current POR
     end_x, end_y = self.getAccVisualPoint(por)
-    
+
     # calculate start_pos
     # get the left, top, right, and bottom coords of the magnifier zoom reg.
     roi_x1, roi_y1, roi_x2, roi_y2 = self.perk.getPos()
@@ -400,19 +404,21 @@ class HandleCaretChange(Task.CaretTask):
   Moves the magnifier to coordinate of this caret
   '''
   def execute(self, por, **kwargs):
-    if not self.perk.perk_state.CaretTrack:
+    if not self.perk_state.CaretTrack:
       return True
+    
     # sanity check
     text_len = self.getAccTextLength()
-    if por.char_offset < 0 or por.item_offset < 0 or \
-       text_len > (por.char_offset + por.item_offset):
+    if (por.char_offset < 0 or 
+        por.item_offset < 0 or
+        text_len < (por.char_offset + por.item_offset)):
       return 
 
-    start_pos, end_pos = self.perk.getDesiredFocalPoint(self.task_por)
+    start_pos, end_pos = self.perk.getDesiredFocalPoint(por)
 
-    if (self.perk.focused_por == self.task_por or 
-        por.item_offset != self.perk.last_caret_offset
-        or self.perk_state.CaretPan):
+    if (self.perk.focused_por == por or 
+        por.item_offset != self.perk.last_caret_offset or 
+        self.perk_state.CaretPan):
       self.perk.goTo(end_pos, start_pos)
     else:
       self.perk.goTo(end_pos)
