@@ -18,6 +18,8 @@ import Perk, Task, AEConstants
 from POR import POR
 import unicodedata
 from i18n import _
+#todo remove me after testing
+import time
 
 __uie__ = dict(kind='perk', all_tiers=True)
 
@@ -33,6 +35,12 @@ class BasicSpeechPerkState(Perk.PerkState):
   CharEcho (bool): Echo characters when typing?
   
   AutoLang (bool): Switch languages automatically?
+
+  CharLimit (numeric): Set the threshold for text insertion synthetizing.
+    If text exceeds this threshold, it will be summarized.
+    
+  @ivar activewincnt: Used to track the number of active windows.  
+  @type last_count: integer
   '''
   def init(self):
     self.newBool('WordEcho', False, _('Echo words?'), 
@@ -48,6 +56,12 @@ class BasicSpeechPerkState(Perk.PerkState):
                  _('When set, the language of the speech synthesizer tries to '
                    'switch to the appropriate language and dialect '
                    'automatically based on the current locale.'))
+    self.newNumeric('CharLimit', 5000, _('Summarize insertions longer than'), 
+                    0, 10000, 0, _('Set the threshold (in characters) for text'
+                    ' insertions. Any text that exceeds this limit will be '
+                    ' summarized by its length.'))
+    # instance variables that are part of state but not a user settings
+    self.activewincnt = 0
 
   def getGroups(self):
     '''
@@ -60,7 +74,7 @@ class BasicSpeechPerkState(Perk.PerkState):
     s = g.newGroup(_('Echo'))
     s.extend(['CharEcho', 'WordEcho'])
     s = g.newGroup(_('Verbosity'))
-    s.extend(['RepeatRole'])
+    s.extend(['RepeatRole', 'CharLimit'])
     s = g.newGroup(_('Language'))
     s.extend(['AutoLang'])
     return g
@@ -102,7 +116,7 @@ class BasicSpeechPerk(Perk.Perk):
     self.setPerkIdealOutput('audio')
     # register event handlers
     self.registerTask(HandleFocusChange('read focus'))
-    self.registerTask(HandleViewChange('read view'))
+    self.registerTask(HandleViewChange('read view',all=True))
     self.registerTask(HandleCaretChange('read caret'))
     self.registerTask(HandleSelectorChange('read selector'))
     self.registerTask(HandleStateChange('read state'))
@@ -134,6 +148,7 @@ class BasicSpeechPerk(Perk.Perk):
     self.registerTask(SpellReviewWord('spell review word'))
     self.registerTask(PronounceReviewWord('pronounce review word'))
     self.registerTask(PronounceReviewChar('pronounce review char'))
+    self.registerTask(MouseToPointer('mouse to pointer'))
     
     # cyclic tasks
     self.registerTask(Task.CyclicInputTask('cycle review char',
@@ -325,7 +340,7 @@ class ReadReviewSkip(Task.InputTask):
   def execute(self, **kwargs):
     if not self.getTempVal('has skipped'):
       # be sure to stop speech if we haven't skipped other items yet
-      self.stopNow()
+      self.stopNow(reset=False)
     self.doTask('read new role')
     
 class ReadReviewWord(Task.InputTask):
@@ -460,68 +475,87 @@ class ReadItemIndex(Task.Task):
     no L{POR} is given. Also speaks the total number of items if it has not
     been announced previously.
     
-    @todo: PP: support radio groups
-    
     @param por: Point of regard to an object possibly having an index of 
       interest
     @type por: L{POR}
     '''
+    por = por or self.task_por
     # get the number of rows and cols
     exts = self.getAccTableExtents(por)
-    if exts is None:
+    if exts:
+      # tables and lists
+      self._handleWithExts(por, exts)
+    else:
+      # try to get special attributes if supported
+      setsize = self.getAccSetSize()
+      pos = self.getAccPosInSet()
       role = self.getAccRole()
-      if role and role.endswith('menu item'):
-        # accessible that does not implement the table interface
-        i = self.getAccIndex(por)
-        # get the parent count
-        parent = self.getParentAcc()
-        count = self.getAccCount(parent)
+      text = None
+      # use special attributes for output if you got them
+      if setsize is not None and pos is not None:
         if not self.perk.last_count:
-          text = (i+1, count)
-          template = _('item %d of %d')
+          text = (pos, setsize)
+          template = _('item %s of %s')
         else:
-          text = i+1
-          template = _('item %d')
-        # first is 1, not 0
+          text = pos
+          template = _('item %s')
+      # use position attribute if that is the only one you got
+      elif pos is not None:
+        text = pos
+        template = _('item %s')
+          
+      # output announcement. Note: items are one-based
+      if text is not None:
         self.sayIndex(text=text, template=template)
         self.perk.last_count = True
-    else:
-      rows, cols = exts
-      # always get the current row number
-      row = self.getAccRow(por)
-      if row is None:
-        # not in a cell
-        return
-      if cols <= 1:
-        # lists
-        if not self.perk.last_count:
-          text = (row+1, rows)
-          template = _('item %d of %d')
-        else:
-          text = row+1
-          template = _('item %d')
-        # first is 1, not 0
-        self.sayIndex(text=text, template=template)
+      
+  def _handleWithExts(self, por, exts):
+    '''
+    Handle cases where extents are present, usually for lists and tables.
+    
+    @param por: Point of regard to an object possibly having an index of 
+      interest
+    @type por: L{POR}
+    @param exts: Extents (rows, cols) of given L{POR}.
+    @type por: tuple
+    '''
+    rows, cols = exts
+    # always get the current row number
+    row = self.getAccRow(por)
+    if row is None:
+      # not in a cell
+      return
+    if cols <= 1:
+      # lists
+      if not self.perk.last_count:
+        text = (row+1, rows)
+        template = _('item %d of %d')
       else:
-        # tables
-        col = self.getAccColumn(por)
-        if not self.perk.last_count:
-          try:
-            text = (row+1, col+1, rows, cols)
-          except TypeError:
-            # not in a cell
-            return
-          template = _('item %d, %d, of %d, %d')
-        else:
-          try:
-            text = (row+1, col+1)
-          except TypeError:
-            # not in a cell
-            return
-          template = _('item %d, %d')
-          # first is 1,1 , not 0,0
-        self.sayIndex(text=text, template=template)
-      self.perk.last_count = True
+        text = row+1
+        template = _('item %d')
+      # first is 1, not 0
+      self.sayIndex(text=text, template=template)
+    else:
+      # tables
+      col = self.getAccColumn(por)
+      if not self.perk.last_count:
+        try:
+          text = (row+1, col+1, rows, cols)
+        except TypeError:
+          # not in a cell
+          return
+        template = _('item %d, %d, of %d, %d')
+      else:
+        try:
+          text = (row+1, col+1)
+        except TypeError:
+          # not in a cell
+          return
+        template = _('item %d, %d')
+        # first is 1,1 , not 0,0
+      self.sayIndex(text=text, template=template)
+    self.perk.last_count = True
+    
 
 class ReadNewHeaders(Task.Task):
   '''
@@ -676,13 +710,15 @@ class ReadAll(Task.InputTask):
   def execute(self, interval=None, **kwargs):
     self.stopNow()
     self.registerTask(_ContinueReadAll('read all continued', 1000))
+    # read the current item again first
+    self.doTask('review current item')
     self.doTask('read all continued')
 
 class _ContinueReadAll(Task.TimerTask):
   '''
   Continue queuing up contents in the view on a timed interval.
   
-  @todo: PP: if index markers are supported, update the start_por accordingly
+  @note: if index markers are supported, update the start_por accordingly
   so a stop leaves the user near the last reviewed location, or maybe this is
   an option?
   
@@ -797,10 +833,6 @@ class ReadTextAttributes(_BaseAttributes):
   '''
   Read all remaining text attribute name/value pairs sans color information.
   
-  @todo: PP: need localizations for all attribute names and text values, or
-    some reasonable subset thereof
-  @todo: PP: probably want names for numbers that have standard levels (e.g. 
-    font weight 400 is bold)
   '''
   def execute(self, **kwargs):
     self.stopNow()
@@ -848,15 +880,28 @@ class ReadItemDetails(Task.Task):
       self.sayValue()
     # i18n hint: a numeric range by a step (e.g. 0.0 to 100.0 by 2.0)
     self.sayValueExtents(template=_('%.1f to %.1f by %.1f'))
+    
+class InViewTimer(Task.TimerTask):
+  '''
+  Executes some time after a View lost event to check for an active window.  
+  Makes an announcement if no window is active.
+  '''
+  def execute(self, **kwargs):
+    # make sure there are still no active windows
+    if self.perk_state.activewincnt == 0:
+      self.sayInfo(text=_('no active window'))
+    self.unregisterTask(self)
 
 class HandleViewChange(Task.ViewTask):
   '''
   Task that handles a L{AEEvent.ViewChange}.
   '''
   def executeLost(self, por, title, **kwargs): 
-    '''
-    @todo: PP: Pause before saying 'No view'
-    '''
+    # decrement active window count 
+    self.perk_state.activewincnt -= 1
+    # start timer if there are no active windows
+    if self.perk_state.activewincnt == 0:
+      self.registerTask(InViewTimer('in view timer', 1500))
     return True
   
   def executeFirstGained(self, por, title, **kwargs):
@@ -876,6 +921,8 @@ class HandleViewChange(Task.ViewTask):
       self.sayWindow(text=_('no title'))
     # reset stateful vars
     self.perk.resetLasts(title)
+    # increment active window count
+    self.perk_state.activewincnt += 1
     return True
   
   def executeGained(self, por, **kwargs):
@@ -891,8 +938,11 @@ class HandleViewChange(Task.ViewTask):
     # prevent the next event (focus?) from stopping this announcement
     self.inhibitMayStop()
     # see if we want to handle this new view in a special way
-    if self.hasAccRole('alert', por):
+    role = self.getAccRole()
+    if role == 'alert':
       self.registerTask(ReadAlert('read alert'))
+    elif role == 'dialog':
+      self.registerTask(ReadDialog('read dialog'))
     return rv
     
 class HandleFocusChange(Task.FocusTask):
@@ -1020,9 +1070,12 @@ class HandleCaretChange(Task.CaretTask):
       c = self.perk_state.CharEcho
       if w and c:
         # word and character echo
+        #s = self.getStyle(layer=AEConstants.LAYER_TIER, 
+                          #sem=AEConstants.SEM_WORD)
         self.mayStop()
         self.sayChar(text=text)
-        self.sayWord(por)
+        self.say(self.getWordText(por), por, sem=AEConstants.SEM_WORD,
+                 layer=AEConstants.LAYER_TIER)
       elif w:
         # word echo
         self.mayStop()
@@ -1032,9 +1085,15 @@ class HandleCaretChange(Task.CaretTask):
         self.mayStop()
         self.sayChar(text=text)
     else:
-      # chunk of text, say the whole thing for now
-      self.mayStop()
-      self.sayItem(text=text)
+      # chunk of text, if length is longer than CharLimit summarize
+      # it (speak text length), else say the whole thing
+      if len(text) > self.perk_state.CharLimit:
+        template=_('%d characters inserted')
+        self.mayStop()
+        self.sayInfo(text=len(text), template=template)
+      else:
+        self.mayStop()
+        self.sayItem(text=text)
     
   def executeMoved(self, por, text, text_offset, **kwargs):
     '''
@@ -1060,7 +1119,6 @@ class HandleCaretChange(Task.CaretTask):
           self.getAccTextSelectionCount() > 0):
         # announce the selection if this is a multiline box as we often do not
         # receive an event; klude that probably breaks for non-gtk apps
-        # @todo: PP: fix me
         self.sayTextAttrs(text=len(self.getAccTextSelection()[0]),
                           template=_('selected %s'))
     elif not por.isSameItem(self.perk.last_caret):
@@ -1189,3 +1247,39 @@ class ReadAlert(Task.FocusTask):
     # unregister this task
     self.unregisterTask(self)
     return True
+      
+class ReadDialog(Task.FocusTask):
+  '''
+  Task registered temporarily when a dialog (not alert) is shown.  Announces 
+  first two labels in the dialog in the order they were traversed.  Traversal 
+  of some containers is bypassed.
+  '''
+  def executeGained(self, **kwargs):
+    count = 0
+    por = self.getViewRootAcc()
+    # start iteration at the root
+    while por is not None:
+      role = self.getAccRole(por)
+      if role == 'label' and count < 2:
+        self.sayItem(por)
+        count += 1
+      elif count >= 2:
+        self.sayInfo(text=_('and other labels'))
+        break
+      # don't traverse some containers
+      if role == 'page tab' or role == 'table' or \
+         role == 'push button' or role == 'combo box':
+        por = self.getNextPeerAcc(por)
+      # get next por walker traverses
+      else:
+        por = self.getNextAcc(por)
+    # unregister this task  
+    self.unregisterTask(self)
+    return True
+      
+class MouseToPointer(Task.InputTask):
+  '''
+  Move the mouse pointer to the task_por (lsr review pointer)
+  '''
+  def execute(self, **kwargs):
+    self.clickPOR(AEConstants.EVENT_SYNTHMOUSE_ABS, por=self.task_por)
