@@ -42,6 +42,9 @@ class FirefoxPerkState(Perk.PerkState):
   boundaries?
   
   AutoManipulable(boolean): When True, sets focus on controls when reviewing.
+  
+  RegConvReview(boolean): When True, automatically registers convenience 
+  review keys.
   '''
   def init(self):
     self.newBool('BoundReview', True, _('Bound review to document?'),
@@ -54,6 +57,14 @@ class FirefoxPerkState(Perk.PerkState):
                 'interactive control while reviewing the document. '
                 'Otherwise, you must manually move focus to interactive '
                 'widgets.'))
+    self.newBool('RegConvReview', True, 
+              _('Review with arrows?'),
+              _('When set, the arrow keys, in conjuction with Caps-Lock, '
+                'will serve as a convenient review tool providing item, word, '
+                'and character review.  When unset, the arrow keys will never '
+                'be registered.  These keys are unregistered outside the '
+                'document and within interactive controls.  Use Caps-Lock/down'
+                ' arrow to toggle this setting.'))
 
   def getGroups(self):
     '''
@@ -65,7 +76,7 @@ class FirefoxPerkState(Perk.PerkState):
     root = self.newGroup()
     root.append('BoundReview')
     root.append('AutoManipulable')
-    
+    root.append('RegConvReview')
     return root
     
 class FirefoxPerk(Perk.Perk):
@@ -120,7 +131,9 @@ class FirefoxPerk(Perk.Perk):
     self.convkeyregistered = False
     self.doc_pors = {}
     self.cached_pointer = None
-    self.last_focus_por = None
+    self.lastporinfocus = None
+    self.lastdocinfocus = None
+    self.viewlost = False
     
     # set an audio device as the default output
     self.setPerkIdealOutput('audio')
@@ -146,6 +159,7 @@ class FirefoxPerk(Perk.Perk):
                                         all=True))
     self.registerTask(ConvReviewFocusReg('firefox conv keys focus reg', 
                                          all=True))  
+    self.registerTask(ConvReviewUserReg('firefox conv keys user reg'))
     
     # Register user triggered move/announcement events
     self.registerTask(MoveToNextHeader('firefox move next header'))
@@ -170,10 +184,12 @@ class FirefoxPerk(Perk.Perk):
                          [kbd.AEK_CAPS_LOCK, kbd.AEK_H])
     self.registerCommand(kbd, 'firefox move active doc', False, 
                          [kbd.AEK_CAPS_LOCK, kbd.AEK_HOME])
+    self.registerCommand(kbd, 'firefox conv keys user reg', False, 
+                         [kbd.AEK_CAPS_LOCK, kbd.AEK_DOWN])
     
     # chain to other tasks.  See Perk docstring for task flowcharts.
     self.chainTask('firefox review role', CHAIN_AROUND, 'read new role')
-    self.chainTask('firefox review should skip', CHAIN_AFTER, 
+    self.chainTask('firefox review should skip', CHAIN_AROUND, 
                    'review should skip')
 
     # register the firefox review task around all the basic review tasks that
@@ -184,6 +200,7 @@ class FirefoxPerk(Perk.Perk):
             name.startswith('review next')) and
             not name.endswith('peek'))):
         self.chainTask('firefox review', CHAIN_AROUND, name)
+        
 
   def getName(self):
     return _('Firefox')
@@ -197,7 +214,7 @@ class FirefoxPerk(Perk.Perk):
     Registers convenience review keys.  Does not attempt to register if
     already registered
     '''
-    if not self.convkeyregistered:
+    if not self.convkeyregistered and self.perk_state.RegConvReview:
       kbd = self.getInputDevice(None, 'keyboard')
       self.registerCommand(kbd, 'firefox convenience review', False, 
                             [kbd.AEK_LEFT])
@@ -243,9 +260,6 @@ class FirefoxPerk(Perk.Perk):
     @type por: L{POR}
     @return: Is por within doc frame?
     @rtype: tuple
-    
-    @todo: SH: replace me.  I'm expensive.  Maybe use bound review task to 
-    cache?
     '''
     if por == None:
       return None
@@ -297,17 +311,14 @@ class FirefoxPerk(Perk.Perk):
     '''
     Determines if a table or cell is for layout only using Firefox's 
     suggestion in the layout-guess attribute.
-    
-    @todo: SH: performace check.  Maybe swap getAccParent instead of 
-    findAccByRole()
-    
+
     @param por: Point of regard to a potential layout table or cell
     @type por: L{POR}
     @param role: Pre-determined role of the L{POR}
     @type role: string    
     '''
     if role == 'table cell':
-      table = self.findAccByRole('table', por=por, ancestor=True)
+      table = self.getParentAcc(por=por)
       if table is None:
         return False
       attrs = self.getAccAttrs(por=table) 
@@ -325,8 +336,10 @@ class HandleAutocomplete(Task.StateTask):
   Announces autocomplete menu opening
   '''
   def execute(self, por, name, value, **kwargs):
-    if name == 'visible' and value == 1 and self.getAccRole(por) == 'menu':
-      self.sayState(text=_('autocomplete menu opened'))
+    if name == 'visible' and value == 1:
+      role = self.getAccRole(por)
+      if role == 'tree' or role == 'tree table':
+        self.sayState(text=_('autocomplete menu opened'))
   
 class ConvReviewViewReg(Task.ViewTask):
   '''
@@ -389,36 +402,11 @@ class HandleDocumentPORView(Task.ViewTask):
   Returns to the last user point of regard in a document after the Firefox 
   window is reactivated.
   '''
-  def executeGained(self, por, **kwargs):
-    '''
-    Tracks the last position reviewed in a document so that it may become the
-    user's L{POR} again after focus is restored to the application.
-    '''
-    # get our current doc frame so we can restore the pointer
-    doc = self.perk.getActiveDocFrame()
-    try:
-      # try to restore it using 'pointer to por' so other perks are notified
-      self.doTask('pointer to por', por=self.perk.doc_pors[doc])  
-    except KeyError, e:
-      # clear cached pointer that is set during caret event
-      self.perk.cached_pointer = None
-      return True
-    
-    # register/unregister convenience keys
-    # do not rely on FirefoxReview because it will cause double announcements
-    if self.perk.needsArrows(self.task_por):
-      self.perk.unregisterConvKeys()
-    else:
-      self.perk.registerConvKeys()
-    
-    # clear cached pointer that is set during caret event
-    self.perk.cached_pointer = None
-    return False
-  
   def executeLost(self, por, **kwargs):
     '''
     Store the pointer using it's doc frame as key.
     '''
+    self.perk.viewlost = True
     doc = self.findAccByRole(DOCUMENT_ROLE, por=self.getPointerPOR(), 
                              ancestor=True)
     if doc is not None:
@@ -439,104 +427,77 @@ class HandleDocumentPORFocus(Task.FocusTask):
     # clear cached pointer that is set during caret event and only used in 
     # executeLost.
     self.perk.cached_pointer = None
-
-    role = self.getAccRole()
-    # handle focus(doc frame) events
-    if role == DOCUMENT_ROLE:
-      try:
-        # no need to restore pointer to itself.  Often the case when 
-        # focus(frame) did restoration.
-        if self.perk.doc_pors[por] == self.getPointerPOR():
-          self.blockNTasks(1, Task.SelectorTask)
-          return False
-      except KeyError:
-        # todo:  Is returning True really needed ??
-        self.blockNTasks(1, Task.SelectorTask)
-        return True
-      
-      # fixes scenerio 5.  Just stay silent here.
-      if por != self.perk.getActiveDocFrame():
-        self.blockNTasks(1, Task.SelectorTask)
-        return False
-      
+    
+    # get a local copy of viewlost.  viewlost is set in view executeLost and 
+    # is used for restoration after an application switch
+    viewlost = self.perk.viewlost
+    self.perk.viewlost = False
+    
+    # get a local copy of the last por that was in focus.  This is used to
+    # help differentiate menu selections from tab/tab-shift events.
+    lastporinfocus = self.perk.lastporinfocus
+    self.perk.lastporinfocus = por
+    
+    # is the user tabbing or tab browsing
+    currentdocinfocus = self.perk.getActiveDocFrame()
+    if self.perk.lastdocinfocus is None or \
+       self.perk.lastdocinfocus != currentdocinfocus:
+      tabbrowsing = True
+    else:
+      tabbrowsing = False
+    self.perk.lastdocinfocus = currentdocinfocus
+    
+    # try to restore the saved POR for all tab browsing events
+    if tabbrowsing:
       # try to restore pointer for this doc frame
       try:
-        self.doTask('pointer to por', por=self.perk.doc_pors[por])
-        # set last_focus_por to indicate focus changed.
-        # last_focus_por used in within doc case to try to differentiate tab
-        # browsing from Tab/Tab-Shift.
-        self.perk.last_focus_por = None
+        # only restore if we are within doc frame
+        if self.perk.getDocFrame(self.task_por):
+          self.doTask('pointer to por',
+                              por=self.perk.doc_pors[currentdocinfocus])
+          # consume the next selector to block selector announcements
+          self.blockNTasks(1, Task.SelectorTask)
+          # register/unregister convenience keys after pointer move
+          self._handleConvReviewReg(self.task_por)
+          # don't propagate task chain
+          return False
+        else:
+          return True
       except KeyError:
-        # todo:  Is returning True really needed ??
-        self.blockNTasks(1, Task.SelectorTask)
+        # let propagated events make announcements
         return True
-      
-      # consume the next selector to block selector announcements
-      self.blockNTasks(1, Task.SelectorTask)
-      # register/unregister convenience keys after pointer move
-      # do not rely on FirefoxReview because it will cause double announcements
-      self._handleConvReviewReg(self.task_por)
-      # don't propagate task chain
-      return False
-    
-    # handle focus(frame) events
-    elif role == 'frame':
-      # try to restore during the focus(frame) event.  focus(doc frame) is
-      # smart enough not to restore again and will silently return.
-      try:
-        self.doTask('pointer to por',  \
-                     por=self.perk.doc_pors[self.perk.getActiveDocFrame()])
-        self._handleConvReviewReg(self.task_por)
-        # set last_focus_por to indicate focus changed.
-        # last_focus_por used in within doc case to try to differentiate tab
-        # browsing from Tab/Tab-Shift.
-        self.perk.last_focus_por = None
-        self.blockNTasks(1, Task.SelectorTask)
-        return False
-      except KeyError:
-        # block selector announcement but let focus announcement through
-        self.blockNTasks(1, Task.SelectorTask)
-        return True
-      
-    # within doc frame
-    elif self.perk.getDocFrame(por):
-      # get the doc frame of the last focus event that was within doc frame.
-      lastfocus_docframe = self.perk.getDocFrame(self.perk.last_focus_por)
-      # update cached por
-      self.perk.last_focus_por = por
-      # catch some tab browsing changes.  Upon startup, we miss annoncement
-      # if control is within doc frame.  Good filter in all other cases seen.
-      if lastfocus_docframe != self.perk.getActiveDocFrame():
-        self.blockNTasks(1, Task.SelectorTask)
-        return False
-      # tabbing (Tab/Tab-Shift) within same doc frame
-      else:
-        # would just returning false be better here ??
-        self.blockNTasks(1, Task.SelectorTask)
-        return True
-      
-    # within chrome  
-    else: 
-      # Let HandleSelectorChange do chrome announcement when pointer and por  
-      # have matching roles, usually for menu items
-      if role == self.getAccRole(self.getPointerPOR()):   
-        return False
-      # Let HandleFocusChange do chrome announcement in other cases.  Usually
-      # occurs when leaving menus and focus is restored to chrome element.  No
-      # selector event occurs here so we must rely on HandleFocusChange to make
-      # announcement.  A tradeoff for not blocking selector is a clipping 
-      # on initial opening of a menu where a selection event does occur.  This 
-      # workaround satisfies scenerio 12. 
-      else:
-        return True
-    
+    # user is hitting tab/tab-shift.    
+    else:
+      lastrole = self.getAccRole(lastporinfocus)    
+   
+      # Try to restore only for certain cases that are misinterpreted
+      # as a tab/tab-shift event.  This includes when a view lost event
+      # occured
+      if viewlost or ((lastrole == 'menu' or lastrole == 'menu item') \
+                       and self.perk.getDocFrame(por)): 
+        try:
+          # only restore if we are within doc frame
+          if self.perk.getDocFrame(self.task_por):
+            self.doTask('pointer to por',
+                                por=self.perk.doc_pors[currentdocinfocus])
+            # consume the next selector to block selector announcements
+            self.blockNTasks(1, Task.SelectorTask)
+            # register/unregister convenience keys after pointer move
+            self._handleConvReviewReg(self.task_por)
+            # don't propagate task chain
+            return False
+          else:
+            return True
+        except KeyError:
+          # let propagated events make announcements
+          return True
+        
   def _handleConvReviewReg(self, por):
     if self.perk.needsArrows(por):
       self.perk.unregisterConvKeys()
     else:
       self.perk.registerConvKeys()
     
-    
   def executeLost(self, por, **kwargs):
     '''
     Store the pointer using it's container doc frame as index if both task_por and
@@ -637,11 +598,6 @@ class FirefoxReview(Task.InputTask):
     return self.getPerkSettingVal('BoundReview')
   
   def execute(self, **kwargs):
-    '''
-    @todo: PP: this method might be refactored, but since so many return values
-      are needed, it made some since to keep it large for the time being;
-      re-evaluate in the future
-    '''
     # start off assuming we're going to call the arounded review task
     original = True
     # start off assuming we're going to let other tasks execute in the chain
@@ -656,20 +612,20 @@ class FirefoxReview(Task.InputTask):
     
     # now determine whether we're 1) in doc to in doc, 2) out of doc to in doc,
     # 3) in doc to out of doc, or 4) out of doc to out of doc
+    
+    # navigating within document or into the document (1 and 2)
     if next_in_doc:
-      # navigating within document or into the document (1 and 2)
       # check if the item should get the focus
-      if self._needsFocus(next_por):
-        # set focus at this location
-        self.setAccPOR(next_por)
-        # do not call the arounded task as we will get a focus event later that
-        # can be announced since we set the focus
-        propagate = False
-      else:
-        # set the focus on the document frame to make later events easier to
-        # handle
+      # edge case where we just entered doc frame from chrome
+      if not in_doc:
+        # set the focus to doc frame to make later events easier to handle
         self.setAccFocus(self.perk.getActiveDocFrame())
-      # check if we need to register/unregister convenience review keys
+      elif self._needsFocus(next_por):
+        # set focus to a manipulable
+        self.setAccPOR(next_por)
+        # inhibit the focus announcement to follow
+        self.inhibitMayStop()
+      # register/unregister convenience review keys 
       if self.perk.needsArrows(next_por):
         self.perk.unregisterConvKeys()
       else:
@@ -694,11 +650,12 @@ class FirefoxReview(Task.InputTask):
         self.perk.unregisterConvKeys()
     # navigating in the chrome (4)
     # do nothing, let the review procede as normal
-    
     if original:
       # execute the anchor, but do not let its immediate chains run too since
       # that would end up re-executing this task ad infinitum
       self.doTask(self.getAnchorTaskId(), chain=False)
+      # moving caret programmatically scrolls screen and brings por into view.
+      self.setAccCaret()
     return propagate
 
 class ReadReviewRole(Task.Task):
@@ -728,15 +685,14 @@ class ReadReviewRole(Task.Task):
       outtext = rolename
         
     if (self.getPerkSettingVal("RepeatRole", 'BasicSpeechPerk') or 
-        outtext != self.getPerkVar("BasicSpeechPerk", "last_role")): 
+        outtext != self.getPerkVar("BasicSpeechPerk", "last_role")):
       if not ((role in self.perk.SKIPPED_ROLES) or
               (role == 'table cell' and self.perk.isLayoutTable(por, role))):
-        # skip the synthesized selector event
-        taskhistory = self.getTaskHistory()
-        if 'read selector' not in taskhistory:
-          self.sayRole(text=outtext)
-      self.setPerkVar("BasicSpeechPerk", "last_role", outtext)
-
+        self.sayRole(text=outtext)
+        # only store new last role if we actually announce it
+        self.setPerkVar("BasicSpeechPerk", "last_role", outtext)
+        
+        
 class ReviewShouldSkip(Task.InputTask):
   '''
   Task is chained after 'review should skip' and provides additional rules to
@@ -747,19 +703,34 @@ class ReviewShouldSkip(Task.InputTask):
   @type por: L{POR}
   '''
   def execute(self, peek, por=None, **kwargs):
-    if self.getTempVal('should skip'):
-      # something already indicated a skip
-      return
-   
-    role = self.getAccRole(por) 
-    if role != '' and role in self.perk.SKIPPED_ITEMS or \
-          role == 'table' and self.perk.isLayoutTable(por, role):
+    text = self.getItemText(por)
+    if text.strip() or self.hasOneAccState(por, 'focusable', 'editable'):
+      # has content or can have content from the user, don't skip
+      self.setTempVal('should skip', False)
+      return 
+    
+    skipping = self.getPerkSettingVal('Skipping', perk_name='ReviewPerk')
+    if skipping == AEConstants.SKIP_ALL:
       self.setTempVal('should skip', True)
-      # don't inform about skips if we're only peeking
+    elif skipping == AEConstants.SKIP_NONE:
+      self.setTempVal('should skip', False)
+    elif skipping == AEConstants.SKIP_REPORT:
       if not peek:
+        role = self.getAccRole(por=por)
+        # then invoke the review report task
+        if role == 'table' and not self.perk.isLayoutTable(por, role):
+          # update the Perk object's task_por
+          self.moveToPOR(por)
+          self.doTask('review skip report')
+        else:
+          self.moveToPOR(por)
         # indicate something has been skipped and reported
-        self.doTask('review skip report')
-        self.setTempVal('has skipped', True)
+        self.setTempVal('has skipped', True)        
+      else:
+        # indicate something will be skipped and reported
+        self.setTempVal('will skip', True)
+      self.setTempVal('should skip', True)
+
     
 class MoveToNextHeader(Task.InputTask):
   '''
@@ -781,6 +752,13 @@ class MoveToNextHeader(Task.InputTask):
         attrs = self.getAccAttrs(nextpor)
         # is it a header?  If so, move pointer, announcement is automatic.
         if attrs.has_key('tag') and len(attrs['tag']) == 2 and attrs['tag'][0] == 'H':
+          # header may be an empty container so look for next item
+          if self.getItemText(nextpor) == '':
+            startpor = nextpor
+            nextpor = self.getNextItem(por=nextpor, wrap=True)
+            # if the next item is empty just revert back to header
+            if self.getItemText(nextpor) == '':
+              nextpor = startpor
           self.doTask('pointer to por', por=nextpor)
           # Register convenience review keys
           self.perk.registerConvKeys()
@@ -808,3 +786,26 @@ class MoveToActiveDoc(Task.InputTask):
     # Register convenience review keys
     self.perk.registerConvKeys()
 
+class ConvReviewUserReg(Task.InputTask):
+  '''
+  Registers/Unregisters convenience review keys on user command.
+    
+  @param por: Point of regard to test
+  @type por: L{POR}
+  '''
+  def execute(self, **kwargs):
+    if self.perk_state.RegConvReview:
+      self.setPerkSettingVal('RegConvReview', False)
+      self.perk.unregisterConvKeys()
+      self.doTask('read message', 
+                  text=_('arrows review off'))
+    else:
+      self.setPerkSettingVal('RegConvReview', True)
+      if (self.perk.getDocFrame(self.task_por)):
+        self.perk.registerConvKeys()
+        self.doTask('read message', 
+                    text=_('arrows review on'))
+      else:
+        self.perk.unregisterConvKeys()
+        self.doTask('read message', 
+                     text=_('arrows review off'))
