@@ -15,8 +15,6 @@ Event announcements
 on activate: ["chat with" username]
 on focus: ["compose" or "history"] <active descendant change>
 on message received: [text of message]
-
-@todo: PP: clean up documentation
   
 @var CONTAINER_PATH: path from frame to container with accessible elements
 @type CONTAINER_PATH: integer
@@ -118,10 +116,14 @@ class GaimPerkState(Perk.PerkState):
     self.newBool('ReadBackground', False, _('Read background messages?'), 
                  _('When checked, incoming messages will be read even when '
                    'Gaim is in the background.'))
+    self.newBool('BrailleBackground', False, _('Braille background messages?'), 
+                 _('When checked, incoming messages will be output to Braille '
+                 'device even when Gaim is in the background.'))
   
   def getGroups(self):
     g = self.newGroup()
     g.append('ReadBackground')
+    g.append('BrailleBackground')
     return g
 
 class GaimPerk(Perk.Perk):
@@ -132,10 +134,7 @@ class GaimPerk(Perk.Perk):
   
   Has initial support for announcing when buddies sign in and out and when the
   status of a conversation changes. Written to work with both gaim 1.5 and 2.0.
-  
-  @todo: DBC: read BuddyList, get ToolTip info per "buddy"
-  @todo: DBC: handle multiple chat windows
-  
+    
   @ivar chat_compose_por: Reference to chat msg compose panel
   @type chat_compose_por: L{POR}
   @ivar chat_history_por: Reference to chat history panel
@@ -165,11 +164,11 @@ class GaimPerk(Perk.Perk):
     # register event handlers
     # events here are app-level, other Perks will handle gnome-level events,
     # so only register events of interest here. 
-    self.registerTask(ActivateGaimWindow())
-    self.registerTask(HandleFocusChange())
-    self.registerTask(HandleSelectorChange())
-    self.registerTask(HandleCaretChange())
-    self.registerTask(ReadIncoming(all=True))
+    self.registerTask(ActivateGaimWindow('gaim read window'))
+    self.registerTask(HandleFocusChange('gaim read focus'))
+    self.registerTask(HandleSelectorChange('gaim read selector'))
+    self.registerTask(HandleCaretChange('game read caret'))
+    self.registerTask(ReadIncoming('gaim read incoming', all=True))
     # not working for background events ???
     #self.registerEventTask(HandlePropertyChange(focus=True, tier=True,
     #                       background=True))
@@ -290,7 +289,7 @@ class GaimPerk(Perk.Perk):
     @return: Human readable name of this L{Perk}.
     @rtype: string
     '''
-    return _('Gaim settings')
+    return _('Gaim')
 
 #
 # Tasks executed by keys
@@ -306,15 +305,12 @@ class ToggleChatPanel(Task.InputTask):
     # determine which chat panel currently has focus.  Specify which 
     # ATSPI-defined property of interest (here, 'focused') and the POR to check
     # for that property
-    # Note: ".perk" lets LSR find cross-class variables "global" to the Perk
     tab_list_por = self.perk.getActivePageTabListPOR()
     if tab_list_por is not None:
       if not self.perk.is_history_focused:
         #change focus to chat history panel
         self.setAccFocus(self.perk.chat_history_por)  
       else:
-        print 'focus to compose'
-        print self.perk.chat_compose_por
         #change focus to chat compose panel
         self.setAccFocus(self.perk.chat_compose_por) 
     return True # allow triggering keystrokes to propagate (default)
@@ -326,6 +322,7 @@ class ConversationStatus(Task.InputTask):
   always reported first.
   '''
   def execute(self, **kwargs):
+    brailleout=[]
     tab_list_por = self.perk.getActivePageTabListPOR()
     # is chat window active? otherwise, don't switch
     if tab_list_por is None:
@@ -345,9 +342,10 @@ class ConversationStatus(Task.InputTask):
       # if acc not an artifact, process
       if name != '':
         value = self.perk.getTypingStatus(por)
+        print selected_tab_por
         if selected_tab_por == por:
-          self.sayState(text=(name, STATUS_MSG[value]), 
-                        template='%s %s,')
+          self.sayState(text=(name, STATUS_MSG[value]), template='%s %s,')
+          brailleout.append('%s: %s, ' % (name, STATUS_MSG[value]))
         else:
           l.append((value, name))
       por = self.getNextPeerAcc(por)
@@ -360,9 +358,11 @@ class ConversationStatus(Task.InputTask):
       if last != value:
         last = value
         self.sayState(text=STATUS_MSG[value], template='%s,')
+        brailleout.append('%s: ' % (STATUS_MSG[value]))
       self.sayState(text=name, template='%s,')
-    #for value, name in l:
-      #self.say('%s %s,' % (name, STATUS_MSG[value]))
+      brailleout.append('%s, ' % (name))
+    # output to Braille device 
+    self.doTask('braille text', text=''.join(brailleout))
 
 #
 # Tasks executed by events
@@ -493,10 +493,6 @@ class ReadIncoming(Task.CaretTask):
     Speaks text that starts with a new line character. This indicates a new
     line of text has been inserted into the message history.
     
-    @todo: PP: should we also check that we're in the chat window? maybe some
-    extension will be inserting text into a passive control? not sure, OK for
-    now probably
-    
     @param por: Point of regard where the caret event occurred
     @type por: L{POR}
     @param text: The text passed during the move
@@ -506,14 +502,31 @@ class ReadIncoming(Task.CaretTask):
     @param layer: Layer of the event, focus, tier, or background
     @type layer: integer
     '''
+    # convoluted logic is necessary in order to minimize expensive  
+    # compareAncestorRoles calls.
     if (not self.perk_state.ReadBackground and 
+        not self.perk_state.BrailleBackground and
         layer == AEConstants.LAYER_BACKGROUND):
-      # don't read background messages if the user has it disabled
+      # don't output background messages if the user has it disabled.
       return
-    if text.startswith('\n'):
+    
+    rv = self.compareAncestorRoles(por, 'scroll pane', 'filler', 'panel',
+                                   'filler', 'split pane', 'filler', 
+                                   'page tab')
+                                   
+    if rv and (layer != AEConstants.LAYER_BACKGROUND or 
+         (layer == AEConstants.LAYER_BACKGROUND and 
+          self.perk_state.ReadBackground)):
       # queue up text to speak
       self.sayItem(text=text)
       
+    if rv and (layer !=  AEConstants.LAYER_BACKGROUND or 
+         (layer == AEConstants.LAYER_BACKGROUND and 
+          self.perk_state.BrailleBackground)):
+      # output to braille device
+      self.doTask('braille text', text=text)
+      
+      
 class HandleCaretChange(Task.CaretTask):
   '''  
   Task that handles a L{AEEvent.CaretChange}. Used to read an entire message in
@@ -599,9 +612,7 @@ class HandleCaretChange(Task.CaretTask):
   
 class HandlePropertyChange(Task.PropertyTask):
   '''  
-  Task that handles a L{AEEvent.PropertyChange}.
-  
-  @todo: DBC: check with gaim 2, with non-gaim2 remote clients
+  Task that handles a L{AEEvent.PropertyChange}.  
   '''  
   def execute(self, por, name, value, **kwargs):
     '''
@@ -650,8 +661,7 @@ class BuddySignOnOff(Task.TableTask):
     '''
     #print "GaimPerk::executeTableRowInserted()", por, first_child_por, \
     #      last_child_por, layer
-    frame_por = self.getRootAcc(por)
-    frame_name = self.getAccName(frame_por)
+    frame_name = self.getWindowTitle(por)
     # only handle table change events from buddy list (don't announce when
     # conversation tab order changes, assume only a sighted user would do that).
     if frame_name == "Buddy List":
@@ -682,8 +692,7 @@ class BuddySignOnOff(Task.TableTask):
     '''
     #print "GaimPerk::executeTableRowDeleted()" , por, first_child_por, \
     #      last_child_por, layer
-    frame_por = self.getRootAcc(por)
-    frame_name = self.getAccName(frame_por)
+    frame_name = self.getWindowTitle(por)
     if frame_name == "Buddy List":
       #print "1" , self.getItemText(first_child_por) 
       #print "2" , self.getItemText(last_child_por) 
