@@ -85,7 +85,7 @@ class Output(Base.TaskTools):
       raise InvalidDeviceError
     return device
     
-  def stopNow(self, dev=None, sem=None, layer=None):
+  def stopNow(self, dev=None, sem=None, layer=None, reset=True):
     '''
     Tells the referenced output device to interrupt current output and clear 
     buffered data.
@@ -98,9 +98,13 @@ class Output(Base.TaskTools):
     @type sem: integer
     @param layer: Layer on which the event occurred
     @type layer: integer
+    @param reset: Reset the L{may_stop} count (True) or leave it as-is when
+    sending the stop?
+    @type reset: boolean
     '''
     dev = dev or self.def_out
-    Output.may_stop = 0
+    if reset:
+      Output.may_stop = 0
     if not self.tier_man.getState().Stopping:
       return
     if layer is None:
@@ -220,9 +224,9 @@ class Output(Base.TaskTools):
     
     This is a generic method which receives all content and commands to be
     rendered and executed on a device. Standard name identifiers should be used
-    whenever possible. For instance, L{AEConstants.CMD_STOP} and
-    L{AEConstants.CMD_TALK}. However, device specific names and values are
-    certainly possible.
+    whenever possible. For instance, L{AEConstants.Output.CMD_STOP} and
+    L{AEConstants.Output.CMD_TALK}. However, device specific names and values
+    are certainly possible.
     
     Only use this method if no other output method is suitable for your 
     purposes.
@@ -365,7 +369,7 @@ class Output(Base.TaskTools):
   def sayTextAttrs(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccAllTextAttrs} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccAllTextAttrs} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -400,7 +404,7 @@ class Output(Base.TaskTools):
   def sayWord(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getWordText} and outputs the result.
+    Calls L{Task.Tools.View.View.getWordText} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -432,7 +436,7 @@ class Output(Base.TaskTools):
   def sayChar(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getCharText} and outputs the result.
+    Calls L{Task.Tools.View.View.getCharText} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -464,7 +468,7 @@ class Output(Base.TaskTools):
   def sayName(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccName} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccName} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -496,7 +500,7 @@ class Output(Base.TaskTools):
   def sayLabel(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccLabel} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccLabel} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -528,7 +532,7 @@ class Output(Base.TaskTools):
   def sayItem(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getItemText} and outputs the result.
+    Calls L{Task.Tools.View.View.getItemText} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -560,7 +564,7 @@ class Output(Base.TaskTools):
   def sayWindow(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getWindowTitle} and outputs the result.
+    Calls L{Task.Tools.View.View.getWindowTitle} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -592,7 +596,7 @@ class Output(Base.TaskTools):
   def saySection(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccLabel} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccLabel} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -624,7 +628,7 @@ class Output(Base.TaskTools):
   def sayApp(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAppName} and outputs the result.
+    Calls L{Task.Tools.View.View.getAppName} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -656,8 +660,8 @@ class Output(Base.TaskTools):
   def sayFont(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccTextAttr} and outputs the value
-  associated with the "family-name" property.
+    Calls L{Task.Tools.View.View.getAccTextAttr} and outputs the value
+    associated with the "family-name" property.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -679,7 +683,7 @@ class Output(Base.TaskTools):
     '''
     if text is None:
       por = por or self.task_por
-      # @todo: DBC: this will not work for all applications, improve this
+      
       text = self.getAccTextAttr('family-name', por)
     if text is None:
       return False
@@ -690,7 +694,7 @@ class Output(Base.TaskTools):
   def sayCount(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccCount} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccCount} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -722,7 +726,7 @@ class Output(Base.TaskTools):
   def sayIndex(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccIndex} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccIndex} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -754,8 +758,8 @@ class Output(Base.TaskTools):
   def sayColor(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccTextAttr} and outputs the value
-  associated with the "fg-color" property.
+    Calls L{Task.Tools.View.View.getAccTextAttr} and outputs the value
+    associated with the "fg-color" property.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -777,7 +781,7 @@ class Output(Base.TaskTools):
     '''
     if text is None:
       por = por or self.task_por
-      # @todo: PP: will not work in all cases, improve
+      
       text = self.getAccTextAttr('fg-color', por)
     if text is None:
       return False
@@ -788,7 +792,7 @@ class Output(Base.TaskTools):
   def sayRole(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccRoleName} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccRoleName} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -820,7 +824,7 @@ class Output(Base.TaskTools):
   def sayLevel(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccLevel} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccLevel} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -852,7 +856,7 @@ class Output(Base.TaskTools):
   def sayStatus(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getItemText} and outputs the result.
+    Calls L{Task.Tools.View.View.getItemText} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -884,7 +888,7 @@ class Output(Base.TaskTools):
   def sayState(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getStateText} and outputs the result.
+    Calls L{Task.Tools.View.View.getStateText} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -916,7 +920,7 @@ class Output(Base.TaskTools):
   def sayDesc(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccDesc} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccDesc} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -948,8 +952,8 @@ class Output(Base.TaskTools):
   def sayHotkey(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccClickKey} and outputs the 
-  result after replacing <> characters.
+    Calls L{Task.Tools.View.View.getAccClickKey} and outputs the 
+    result after replacing <> characters.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -1084,7 +1088,7 @@ class Output(Base.TaskTools):
   def sayValue(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccFloatValue} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccFloatValue} and outputs the result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -1116,7 +1120,8 @@ class Output(Base.TaskTools):
   def sayValueExtents(self, por=None, text=None, template=u'%s', 
                   dev=None, talk=True):
     '''
-    Calls L{Task.Tools.View.getAccFloatValueExtents} and outputs the result.
+    Calls L{Task.Tools.View.View.getAccFloatValueExtents} and outputs the
+    result.
   
     @param por: L{POR} pointing at an accessible whose value will be read or
       None when the L{task_por} should be used. The value is read only if the
@@ -1265,8 +1270,6 @@ class Output(Base.TaskTools):
     @type layer: integer
     @param dev: Device that should receive the command
     @type dev: L{AEOutput}
-    @param talk: Send a talk command immediately following the command?
-    @type talk: boolean
     @return: IANA tag name closest to the one given or None meaning no 
       supported language makes sense
     @rtype: string
