@@ -94,6 +94,7 @@ class Tier(object):
     self.pointer_por = por
     self.focus_por = None
     self.last_key = (None, None, None)
+    self.auto_pointer = True
     # fields to be cleared after each event
     self.clearState()
 
@@ -120,6 +121,24 @@ class Tier(object):
     @type por: L{POR}
     '''
     self.pointer_por = por
+    
+  def getAutoPointer(self):
+    '''
+    Gets the auto pointer mode for this L{Tier}.
+    
+    @return: Current pointer mode.
+    @rtype: boolean
+    '''
+    return self.auto_pointer
+  
+  def setAutoPointer(self, mode):
+    '''
+    Sets the auto pointer mode for this L{Tier}.
+    
+    @param mode: Mode to set.
+    @type mode: boolean
+    '''
+    self.auto_pointer = mode
 
   def getFocus(self):
     '''
@@ -511,8 +530,8 @@ class Tier(object):
       L{Task.Base.Task.update} instead for housekeeping?
     @type propagate: boolean
     @param layer: Layer on which the L{AEEvent} occurred, one of 
-      L{AEConstants.LAYER_FOCUS}, L{AEConstants.LAYER_TIER}, 
-      L{AEConstants.LAYER_BACKGROUND}
+      L{AEConstants.Event.LAYER_FOCUS}, L{AEConstants.Event.LAYER_TIER}, 
+      L{AEConstants.Event.LAYER_BACKGROUND}
     @type layer: integer
     @param params: Keyword arguments to be provided to the L{Task} as 
       parameters
@@ -577,7 +596,10 @@ class Tier(object):
     '''
     try:
       # grab POR from any focus type event
-      self.focus_por = event.getFocusPOR()
+      if self.auto_pointer:
+        self.focus_por = event.getFocusPOR()
+      else:
+        self.focus_por = self.getPointer()
     except AttributeError:
       pass
     
@@ -681,8 +703,8 @@ class Tier(object):
     @param name: Name of the L{Task} to execute
     @type name: string
     @param layer: Layer in which to execute the L{Task}, one of 
-      L{AEConstants.LAYER_FOCUS}, L{AEConstants.LAYER_TIER}, 
-      L{AEConstants.LAYER_BACKGROUND}
+      L{AEConstants.Event.LAYER_FOCUS}, L{AEConstants.Event.LAYER_TIER}, 
+      L{AEConstants.Event.LAYER_BACKGROUND}
     @type layer: integer
     @param task_params: Keyword data to pass to the L{Task} as parameters on 
       execution or update
